<?php
// Copyright Brian Quade, BQ Plugins 2012, bqplugins.com

if (!defined ('ABSPATH'))
	die ();

require_once 'bq-download-model.php';
require_once 'bq-download-util.php';

class BQChartPeriod
{
	const cookie_name = 'bq_download_chart_period';

//'hour', 'day', 'week', 'month'
	protected static $period_list = array ();

	protected $name; // name of this chart
	protected $period;
	protected $end_date = '0000-00-00 00:00:00'; // defaults to current date

	// must be called before headers are sent
	// item_max is the number of items in this sorter list
	public function __construct ($name)
	{
		$this->name = $name;

		$this->read_cookie ();
	}

	// sets the data members and fills the static array if it is still empty
	protected function read_cookie ()
	{
		// if this page is already set in the static list, use those values to initialize this class
		if (isset (self::$period_list [$this->name]))
		{
			$this->period = self::$period_list [$this->name] ['period'];
//			$this->end_date = self::$period_list [$this->name] ['end_date'];
			return;
		}

		// read the cookie
		$raw_cookie = BQDownloadUtil::bq_get_cookie (self::cookie_name);
		$cookie_was_empty = $raw_cookie === false;

		if (!$cookie_was_empty)
		{
			// read the cookie into memory
			$sorter_settings = explode ('|', $raw_cookie);
			foreach ($sorter_settings as $settings)
			{
				list ($name, $period, $end_date) = explode (',', $settings);
				self::$period_list [$name] = array ('period' => $period, 'end_date' => $end_date);

				if (strcmp ($name, $this->name) == 0)
					$this->set_values ($period, $end_date, false);
			}

			// if the settings were not found in the cookie, add default ones
			if (!isset (self::$period_list [$this->name]))
				$this->add_new_chart ();
		}
		else // the cookie was empty
			$this->add_new_chart ();
	}

	// writes the cookie from the static array
	protected function write_cookie ()
	{
		if (headers_sent ())
			return false;

		// construct the cookie string
		$cookie_str = '';
		foreach (self::$period_list as $name => $settings)
		{
			if (strlen ($cookie_str) > 0)
				$cookie_str .= '|';
			$cookie_str .= $name.','.$settings ['period'].','.$settings ['end_date'];
		}

		// write the cookie
		BQDownloadUtil::bq_set_cookie (self::cookie_name, $cookie_str);

		return true;
	}

	protected function add_new_chart ()
	{
		$this->set_values ('day');
		return true;
	}

	protected function set_values ($period, $end_date = '0000-00-00 00:00:00', $write = true)
	{
		// some items might have been removed since cookie was last updated
		$this->period = $period;
		$this->end_date = $end_date;

		// update the static array
		self::$period_list [$this->name] = array ('period' => $this->period, 'end_date' => $this->end_date);

		return !$write || $this->write_cookie ();
	}

	public function get_period ()
	{
		return $this->period;
	}

	// returns true if the new settings were saved in the cookie
	public function set_period ($value)
	{
		return $this->set_values ($value, $this->end_date);
	}

	public function get_end_date ()
	{
		return $this->end_date;
	}

	// returns true if the new settings were saved in the cookie
	public function set_end_date ($value)
	{
		return $this->set_values ($this->period, $value);
	}

	public function is_current_date ()
	{
		return strcmp ($this->end_date, '0000-00-00 00:00:00') == 0;
	}

/*
	public function display_control ()
	{
	}
*/

	public static function expire ()
	{
		BQDownloadUtil::bq_delete_cookie (self::cookie_name);
	}
}
?>
