<?php
// Copyright Brian Quade, BQ Plugins 2012, bqplugins.com

if (!defined ('ABSPATH'))
	die ();

require_once 'bq-download-model.php';
require_once 'bq-download-util.php';
require_once 'bq-pager.php';
require_once 'bq-sorter.php';
require_once 'bq-chart-period.php';

class BQDownloadAdmin
{
	public static function ajax_admin ()
	{
//		header ('Content-Type: text/plain; charset=utf-8;');
		header ('Cache-Control: no-cache');

		if (!isset ($_POST ['name']) || empty ($_POST ['name']) || !isset ($_POST ['f']) || empty ($_POST ['f']))
			die;

		// with last parameter set to true, this will die before the function returns if the nonce is incorrect
		check_ajax_referer ($_POST ['name'], 'nonce', true);

		$response = new stdClass ();
		$function = $_POST ['f'];

		// perform the action
		switch ($function)
		{
			case 'delete_file':
				if (!isset ($_POST ['file']) || empty ($_POST ['file']))
					die;
				$file_id = $_POST ['file'];
				$file_name = BQDownloadModel::get_file_name_by_id ($file_id);
				if (empty ($file_name))
					$response->error = 'That file does not exist in the database.';
				else
				{
					if (BQDownloadModel::delete_file ($file_id) !== false)
					{
						$response->file_list = BQDownloadUtil::get_file_list_window ();
						$response->message = $file_name.' has been permanently deleted.';
					}
					else
						$response->error = 'Failed to delete '.$file_name.'.';
				}
				break;

			case 'get_stats':
				if (!isset ($_POST ['period']) || empty ($_POST ['period']))
					die;

				$file_downloads_chart = new BQChartPeriod ('file_downloads');
				$file_downloads_chart->set_period ($_POST ['period']);

				$response->stats = BQDownloadUtil::get_download_bar_chart ();
				break;

			case 'get_files':
				$key = $_POST ['key'];
				$value = $_POST ['v'];

				$file_pager = new BQPager ('file', BQDownloadModel::get_file_count ());

				// get the files here so that the same list will be used to calculate new start and display the list
				switch ($key)
				{
					case 'page':
						$file_pager->set_start ($value);
						break;
					case 'count':
						$file_pager->set_count ($value);
						break;
				}
				$response->files = BQDownloadUtil::get_file_list_window ();
				break;

			case 'sort_files':
				$key = $_POST ['key'];
				$dir = $_POST ['dir'];
				$file_sorter = new BQSorter ('file');
				$file_sorter->set_field ($key);
				$file_sorter->set_asc (strcasecmp ($dir, 'asc') == 0);
				$response->files = BQDownloadUtil::get_file_list_window ();
				break;

			case 'get_emails':
				$key = $_POST ['key'];
				$value = $_POST ['v'];

				$email_pager = new BQPager ('email', BQDownloadModel::get_user_count ());

				// get the files here so that the same list will be used to calculate new start and display the list
				switch ($key)
				{
					case 'page':
						$email_pager->set_start ($value);
						break;
					case 'count':
						$email_pager->set_count ($value);
						break;
				}
				$response->emails = BQDownloadUtil::get_email_list_window ();
				break;

			case 'sort_emails':
				$key = $_POST ['key'];
				$dir = $_POST ['dir'];
				$email_sorter = new BQSorter ('email');
				$email_sorter->set_field ($key);
				$email_sorter->set_asc (strcasecmp ($dir, 'asc') == 0);
				$response->emails = BQDownloadUtil::get_email_list_window ();
				break;

			case 'save_settings':
				$return_email = $_POST ['re'];
				BQDownloadUtil::bq_update_option ('bq_download_return_email', $return_email);
				$response->success = true;
				$response->message = 'Settings were saved successfully';
				break;
		}

		die (json_encode ($response));
	}
}
?>
