<?php
// Copyright Brian Quade, BQ Plugins 2012, bqplugins.com

if (!defined ('ABSPATH'))
	die ();

require_once 'bq-download-util.php';
require_once 'bq-pager.php';
require_once 'bq-pager.php';
require_once 'bq-sorter.php';
require_once 'bq-chart-period.php';

// $bq_download_db_schema_version
require_once 'bq-download-db-schema.php';

class BQDownloadModel
{
	protected static $db_updated = false;


	// database maintenance functions

	public static function get_current_db_version ()
	{
		global $bq_download_db_schema_version;
		return $bq_download_db_schema_version [count ($bq_download_db_schema_version) - 1] ['version'];
	}

	public static function update_database ()
	{
		if (self::$db_updated)
			return true;

		global $bq_download_db_schema_version;
		global $wpdb;

		// update database schema
		$this_db_version = BQDownloadUtil::bq_get_option ('bq_download_db_version', 1);
		$current_db_version = self::get_current_db_version ();
		$failed = false;
		if ($this_db_version === false || strcmp ($this_db_version, $current_db_version) != 0)
		{
			$db_uninitialized = $this_db_version === false;
			$cur_version_found = false;
			foreach ($bq_download_db_schema_version as $schema)
			{
				// interate through the version until the current one is found
				if (!$db_uninitialized && !$cur_version_found)
				{
					$cur_version_found = strcmp ($this_db_version, $schema ['version']) == 0;
					continue;
				}

				// after the current version is found, do each sql statement for subsequent versions
				foreach ($schema ['sql'] as $sql)
				{
					$failed = $wpdb->query ($sql) === false;
					if ($failed)
						break;
				}

				// update the option if the database was updated successfully
				if (!$failed)
				{
					if ($this_db_version === false && strcmp ($schema ['version'], $bq_download_db_schema_version [0] ['version']) == 0)
						BQDownloadUtil::bq_add_option ('bq_download_db_version', $schema ['version'], 1); // first activation
					else
						BQDownloadUtil::bq_update_option ('bq_download_db_version', $schema ['version'], 1); // upgrade
				}
				else
					break; // it failed, stop right here
			}
		}

		self::$db_updated = !$failed;
		return self::$db_updated;
	}

	// only called from uninstall.php
	public static function remove_database ()
	{
		global $wpdb;
		$wpdb->query ('drop table `bq_download_file`');
		$wpdb->query ('drop table `bq_download_user`');
		$wpdb->query ('drop table `bq_download`');
		BQDownloadUtil::bq_delete_option ('bq_download_db_version', 1);
	}

	public static function get_mysql_version ()
	{
		global $wpdb;
		$sql = 'select @@version';
		return $wpdb->get_var ($sql); // returns NULL on error, string on success
	}

/*
	public static function remove_blog ($blog_id = null)
	{
		global $wpdb;

		if ($blog_id === null)
			$blog_id = BQDownloadUtil::get_blog_id ();

		// delete the records specific to this blog
		$where_clause = $blog_id !== null ? 'blog_id = '.$blog_id : 'isnull(blog_id)';
		$sql = 'update bq_download_file set deleted = true where '.$where_clause;
//		$sql = 'delete from bq_download '.
//				'where file_id in '.
//						'(select distinct id from bq_download_file where '.$where_clause.')';
		$wpdb->query ($sql);

//		$where_clause = $blog_id !== null ? 'blog_id = '.$blog_id : 'isnull(blog_id)';
//		$sql = 'delete from `bq_download_file` where '.$where_clause;
//		$wpdb->query ($sql);

		// remove the file directory
//		BQDownloadUtil::remove_download_dir ($blog_id);
	}
*/

	public static function get_multisite_list ()
	{
		$blog_list = array ();

		if (is_multisite ())
		{
			global $wpdb;
			$sql = 'select blog_id from '.$wpdb->blogs.' where site_id = '.$wpdb->siteid.' and spam = 0 and deleted = 0 and archived = "0"';
			$results = $wpdb->get_results ($sql, ARRAY_A);
			$blog_list = array ();
			foreach ($results as $blog_result)
				$blog_list [] = (int) $blog_result ['blog_id'];
		}

		return $blog_list;
	}

	// returns a list of blogs that currently have non-deleted files in the database
	public static function get_db_blog_list ()
	{
		$blog_list = array ();

		if (is_multisite ())
		{
			global $wpdb;
			$sql = 'select distinct blog_id from bq_download_file';
			$results = $wpdb->get_results ($sql, ARRAY_A);
			$blog_list = array ();
			foreach ($results as $blog_result)
				$blog_list [] = (int) $blog_result ['blog_id'];
		}

		return $blog_list;
	}

	// file functions

	// removes files from the database that have been manually deleted from the download directory
	// adds files to the database have have been manually uploaded to the download directory
	public static function synch_data ()
	{
		global $wpdb;

		// create a list of name of all files in the directory
		$path_to_files = BQDownloadUtil::get_download_path ();
		$all_dir_files = array (); // all files in the directory
		$new_files = array (); // files in the directory that are not in the database

		// create a list of ids and names and all files in the database
		$blog_id = BQDownloadUtil::get_blog_id ();
		$where_clause = 'where deleted = false'.($blog_id !== null ? ' and blog_id = '.$blog_id : '');
		$sql = 'select id, name from bq_download_file '.$where_clause;
		$all_db_files = $wpdb->get_results ($sql, OBJECT); // returns an empty array on error

		$handle = opendir ($path_to_files);
		if ($handle !== false)
		{
			// create a list of names of all files in the directory but not in the database
			$dir_file = readdir ($handle);
			while ($dir_file !== false)
			{
				if (!is_dir ($path_to_files.'/'.$dir_file))
				{
					$all_dir_files [] = $dir_file;

					$found = false;
					foreach ($all_db_files as $db_file)
					{
						if (strcmp ($db_file->name, $dir_file) == 0)
						{
							$found = true;
							break;
						}
					}
					if (!$found)
						$new_files [] = $dir_file;
				}
				$dir_file = readdir ($handle);
			}

			closedir ($handle);
		}

		// add the new files
		self::add_multiple_files ($new_files);

		// remove the deleted files
		$removed_files = array ();
		foreach ($all_db_files as $db_file)
		{
			// add it to a separate file list for comparison with the directory listing
			if (!in_array ($db_file->name, $all_dir_files))
				$removed_files [] = $db_file->id;
		}
		self::delete_multiple_files ($removed_files);

		return true;
	}

	public static function get_paged_files ()
	{
		// synching the database with the reality of the file directory, which could have changed
		self::synch_data ();

		// pager stuff
		$file_pager = new BQPager ('file', self::get_file_count ());
		$limit_clause = $file_pager->get_limit_clause ();

		// sorter stuff
		$file_sorter = new BQSorter ('file');
		$sort = $file_sorter->get_field ();
		$sort_dir = $file_sorter->get_asc () ? 'asc' : 'desc';

		// where clause
		$blog_id = BQDownloadUtil::get_blog_id ();
		$where_clause = 'where bq_download_file.deleted = false and ';
		$where_clause .= $blog_id !== null ? 'bq_download_file.blog_id = '.$blog_id : 'isnull(bq_download_file.blog_id)';

		// order by clauses
		$sort_clause_1 = '';
		$sort_clause_3 = '';
		switch ($sort)
		{
			case 'name':
				$sort_clause_1 = $file_sorter->get_order_by_clause ();
				break;
			case 'size':
			case 'last_uploaded':
				// these fields will be sorted and paged manually later
				$sort_clause_1 = 'order by name '.$sort_dir; // name as a secondary sort field
				$limit_clause = ''; // we need to sort all files by size and last_uploaded programatically before applying limit
				break;
			case 'last_downloaded':
			case 'download_count':
				$sort_clause_3 = $file_sorter->get_order_by_clause ();
				break;
		}

		// returns: id, name
		// for all files owned by this blog
		$file_query =
				'select id, name '.
				'from bq_download_file '.
				$where_clause.' '.
				$sort_clause_1;

		// returns: file_id, last_downloaded, download_count
		// for all files owned by this blog that have been downloaded
		$download_query =
				'select file_id, max(time_downloaded) as last_downloaded, count(time_downloaded) as download_count '.
				'from bq_download '.
					'inner join bq_download_file '.
					'on bq_download_file.id = bq_download.file_id '.
				$where_clause.' '.
				'group by file_id';

		$sql = 'select file_query.id as id, file_query.name as name, download_query.last_downloaded as last_downloaded, download_query.download_count as download_count '.
				'from ('.$file_query.') as file_query '.
					'left join ('.$download_query.') as download_query '.
					'on file_query.id = download_query.file_id '.
				$sort_clause_3.' '.
				$limit_clause;
		global $wpdb;
		$db_files = $wpdb->get_results ($sql, OBJECT); // returns an empty array on error

		// get the fields that were not supplied by the database
		$file_count = count ($db_files);
		$path_to_files = BQDownloadUtil::get_download_path ();
		for ($f = 0; $f < $file_count; $f++)
		{
			$path = $path_to_files.'/'.$db_files [$f]->name;
			$db_files [$f]->size = filesize ($path);
			$db_files [$f]->last_uploaded = date ('Y-m-d H:i:s', BQDownloadUtil::convert_to_timezone (filemtime ($path), 'UTC', ini_get ('date.timezone')));
		}

		// sort on any fields that were calculated and could not be sorted in the database query
		switch ($sort)
		{
			case 'size':
			case 'last_uploaded':
				$db_files = $file_sorter->manual_sort ($db_files);
				if (!$file_pager->is_all ())
					$db_files = $file_pager->manual_limit ($db_files);
				break;
		}

		// get some more text fields for the files that will be displayed
		$file_count = count ($db_files); // file count may have changed
		for ($f = 0; $f < $file_count; $f++)
		{
			$file_strings = self::get_file_strings ($db_files [$f]);
			$db_files [$f]->last_uploaded_text = $file_strings ['last_uploaded_text'];
			$db_files [$f]->last_uploaded_title = $file_strings ['last_uploaded_title'];
			$db_files [$f]->last_downloaded_text = $file_strings ['last_downloaded_text'];
			$db_files [$f]->last_downloaded_title = $file_strings ['last_downloaded_title'];
			$db_files [$f]->size_text = $file_strings ['size_text'];
			$db_files [$f]->size_title = $file_strings ['size_title'];
		}

		return $db_files;
	}

	// returns the total number of files in the database for this blog
	public static function get_file_count ()
	{
		global $wpdb;

		$blog_id = BQDownloadUtil::get_blog_id ();
		$where_clause = 'where deleted = false and '.($blog_id !== null ? 'blog_id = '.$blog_id : 'isnull(blog_id)');
		$sql = 'select count(id) from bq_download_file '.$where_clause;
		$file_count = $wpdb->get_var ($sql); // returns NULL on error, string on success

		return $file_count !== null ? (int) $file_count : false;
	}

	public static function add_file ($name)
	{
/*
		$forbidden_chars = array ('/', "\\", '?', '%', '*', ':', '|', '"', "'", '<', '>');
		$name_okay = true;

		foreach ($forbidden_chars as $test)
		{
			if (strpos ($name, $test) !== false)
			{
				$name_okay = false;
				break;
			}
		}
		if (!$name_okay)
			return false;
*/

		if (!is_string ($name) || strlen ($name) == 0)
			return false;

		global $wpdb;
		$filename = $wpdb->_escape ($name);

		$blog_id = BQDownloadUtil::get_blog_id ();
		if ($blog_id !== null)
			$sql = 'insert into `bq_download_file` set blog_id = '.$blog_id.', name = "'.$filename.'"';
		else
			$sql = 'insert into `bq_download_file` set name = "'.$filename.'"';
		$rows_affected = $wpdb->query ($sql);
		if ($rows_affected === false)
			return false;

		return $wpdb->insert_id;
	}

	protected static function add_multiple_files ($files)
	{
		if (!is_array ($files) || count ($files) == 0)
			return false;

		$blog_id = BQDownloadUtil::get_blog_id ();
		$blog_id_value = $blog_id !== null ? $blog_id : 'null';

		global $wpdb;

		$new_files = '';
		foreach ($files as $file)
		{
			if (strlen ($new_files) == 0)
				$new_files .= '("';
			else
				$new_files .= '",'.$blog_id_value.'),("';
			$new_files .= $wpdb->_escape ($file);
		}
		$new_files .= '",'.$blog_id_value.')';

		$sql = 'insert into bq_download_file (name, blog_id) values '.$new_files;
		$rows_affected = $wpdb->query ($sql);
		if ($row_affected === false)
			return false;

		return true;
	}

	// deletes the file from the directory
	public static function delete_file ($id)
	{
		$filename = self::get_file_name_by_id ($id);
		if (empty ($filename))
			return false;

		// delete the file from the directory
		$path_to_files = BQDownloadUtil::get_download_path ();
		$full_path = realpath ($path_to_files.'/'.$filename);
		if (file_exists ($full_path))
		{
			if (!unlink ($full_path))
				return false;
		}

		global $wpdb;

		// delete the file from the database
		$sql = 'update bq_download_file set deleted = true where id = '.$id;
		$rows_affected = $wpdb->query ($sql);
		if ($rows_affected === false)
			return false;

		return true;
	}

	// files is an array of files ids
	// does not remove files from the directory
	public static function delete_multiple_files ($files)
	{
		if (!is_array ($files) || count ($files) == 0)
			return false;

		global $wpdb;

		$remove_files = '';
		foreach ($files as $file)
		{
			if (strlen ($remove_files) == 0)
				$remove_files .= '(';
			else
				$remove_files .= ',';
			$remove_files .= $wpdb->_escape ($file);
		}
		$remove_files .= ')';

		$sql = 'update bq_download_file set deleted = true where id in '.$remove_files;
		$rows_affected = $wpdb->query ($sql);
		if ($rows_affected === false)
			return false;

		return true;
	}

	protected static function get_file_strings ($file)
	{
		$file_strings = array ();

		// create the text for the date and time fields
		$last_uploaded = BQDownloadUtil::get_wp_date_time ($file->last_uploaded);
		if (!empty ($last_uploaded))
		{
			$file_strings ['last_uploaded_text'] = $last_uploaded ['date'];
			$file_strings ['last_uploaded_title'] = 'Uploaded: '.$last_uploaded ['date'].' at '.$last_uploaded ['time'];
		}
		else
		{
			$file_strings ['last_uploaded_text'] = '';
			$file_strings ['last_uploaded_title'] = '';
		}

		$last_downloaded = BQDownloadUtil::get_wp_date_time ($file->last_downloaded);
		if (!empty ($last_downloaded ['date']) && !empty ($last_downloaded ['time']))
		{
			$file_strings ['last_downloaded_text'] = $last_downloaded ['date'];
			$file_strings ['last_downloaded_title'] = 'Last Downloaded: '.$last_downloaded ['date'].' at '.$last_downloaded ['time'];
		}
		else
		{
			$file_strings ['last_downloaded_text'] = '';
			$file_strings ['last_downloaded_title'] = '';
		}

		// make the file sizes pretty
		$kilobyte = 1024;
		$megabyte = 1048576;
		if ($file->size < $kilobyte)
			$file_strings ['size_text'] = $file->size;
		else if ($file->size < $megabyte)
			$file_strings ['size_text'] = round ($file->size / $kilobyte).'K';
		else
			$file_strings ['size_text'] = round ($file->size / $megabyte, 1).'M';
		$file_strings ['size_title'] = $file->size.' bytes';

		return $file_strings;
	}

	public static function get_file_id_by_name ($filename)
	{
		if (!is_string ($filename))
			return false;

		global $wpdb;

		$filename = $wpdb->_escape ($filename);

		$sql = 'select id from bq_download_file where deleted = false and name = "'.$filename.'"';
		$blog_id = BQDownloadUtil::get_blog_id ();
		if ($blog_id !== null)
			$sql .= ' and blog_id = '.$blog_id;

		$file_record = $wpdb->get_results ($sql, OBJECT); // returns an empty array on error
		if (empty ($file_record) || count ($file_record) == 0 || !isset ($file_record [0]->id))
			return false;

		return $file_record [0]->id;
	}

	public static function get_file_name_by_id ($file_id)
	{
		global $wpdb;

		$file_id = (int) $file_id;

		$sql = 'select name from bq_download_file where id = '.$file_id;
		$blog_id = BQDownloadUtil::get_blog_id ();
		if ($blog_id !== null)
			$sql .= ' and blog_id = '.$blog_id;

		$file_record = $wpdb->get_results ($sql, OBJECT); // returns an empty array on error
		if (empty ($file_record) || count ($file_record) == 0 || !isset ($file_record [0]->name))
			return false;

		return $file_record [0]->name;
	}

	// email user functions

	public static function get_users ()
	{
		global $wpdb;

		// read the files from the database
		$blog_id = BQDownloadUtil::get_blog_id ();
		if ($blog_id !== null)
			$where_clause = 'and blog_id = '.$blog_id.' ';
		else
			$where_clause = '';

		$email_pager = new BQPager ('email', self::get_user_count ());
		$start = $email_pager->get_start () - 1;
		$count = $email_pager->get_count ();

		$email_sorter = new BQSorter ('email');
		$sort = $email_sorter->get_field ();
		$sort_dir = $email_sorter->get_asc () ? 'asc' : 'desc';

		// a count of 0 means get all files
		$limit_clause = $email_pager->get_limit_clause ();

		// prepare order by clauses for sort criteria that can be sorted in the query
		$q3_sort_clause = '';
		switch ($sort)
		{
			case 'email':
				$q3_sort_clause = $email_sorter->get_order_by_clause ().' ';
				break;
			case 'request_count':
				$q3_sort_clause = $email_sorter->get_order_by_clause ().', download_count desc ';
				break;
			case 'download_count':
				$q3_sort_clause = $email_sorter->get_order_by_clause ().', request_count desc ';
				break;
		}

		$sql = 'select bq_download_user.email as email, q3.request_count as request_count, if(not isnull(q3.download_count),q3.download_count,0) as download_count '.
				'from bq_download_user '.
					'inner join ('.
						'select q1.user_id as user_id, q1.request_count as request_count, q2.download_count as download_count '.
						'from ('.
								'select user_id, count(time_requested) as request_count '.
								'from bq_download '.
									'inner join bq_download_file '.
									'on bq_download_file.id = bq_download.file_id '.
								'where not isnull(bq_download.user_id) '.$where_clause.
								'group by user_id'.
							') as q1 '.
							'left join ('.
								'select user_id, count(time_downloaded) as download_count '.
								'from bq_download '.
									'inner join bq_download_file '.
									'on bq_download_file.id = bq_download.file_id '.
								'where not isnull(bq_download.user_id) and bq_download.time_downloaded <> "0000-00-00 00:00:00" '.$where_clause.
								'group by user_id'.
							') as q2 '.
							'on q1.user_id = q2.user_id'.
					') as q3 '.
					'on bq_download_user.id = q3.user_id '.
				$q3_sort_clause.
				$limit_clause;

		$db_users = $wpdb->get_results ($sql, OBJECT); // returns an empty array on error

		return $db_users;
	}

	public static function get_user_count ()
	{
		global $wpdb;
		$blog_id = BQDownloadUtil::get_blog_id ();
		$sql = 'select count(distinct user_id) '.
				'from bq_download '.
					'inner join bq_download_file '.
					'on bq_download_file.id = bq_download.file_id '.
					($blog_id !== null ? 'and bq_download_file.blog_id = '.$blog_id.' ' : '').
				'where not isnull(bq_download.user_id)';
		$user_count = $wpdb->get_var ($sql); // returns NULL on error, string on success
		return $user_count !== null ? (int) $user_count : false;
	}

	// returns id of email address, or false
	public static function add_user ($email)
	{
		if (!is_string ($email) || strlen ($email) == 0)
			return false;

		$user_id = self::get_user_id_by_email ($email);
		if ($user_id !== false) // it already exists
			return $user_id;

		global $wpdb;
		$email_address = $wpdb->_escape ($email);
		$sql = 'insert into `bq_download_user` set email = "'.$email_address.'"';
		$wpdb->query ($sql);
		return $wpdb->insert_id;
	}

	public static function get_user_id_by_email ($email)
	{
		if (strcmp (gettype ($email), 'string') != 0)
			return false;

		global $wpdb;

		// get the user id
		$sql = 'select id from bq_download_user where email = "'.$wpdb->_escape ($email).'"';
		$result = $wpdb->get_results ($sql, ARRAY_A);
		if (count ($result) == 0 || !isset ($result [0] ['id']) || empty ($result [0] ['id']))
			return false;

		return $result [0] ['id'];
	}

	public static function get_email_by_user_id ($id)
	{
		global $wpdb;

		// get the user email
		$sql = 'select email from bq_download_user where id = "'.$wpdb->_escape ($id).'"';
		$result = $wpdb->get_results ($sql, ARRAY_A);
		if (count ($result) == 0 || !isset ($result [0] ['email']) || empty ($result [0] ['email']))
			return false;

		return $result [0] ['email'];
	}

	// download and stats functions

	// add a record with a nonce (download_key) to the bq_download table
	public static function add_download_request ($email, $filename, $nonce)
	{
		global $wpdb;

		$user_id = self::get_user_id_by_email ($email);
		if ($user_id === false)
		{
			$user_id = self::add_user ($email);
			if ($user_id === false)
				return false;
		}

		$file_id = self::get_file_id_by_name ($filename);
		if ($file_id === false)
			return false;

		$sql = 'insert into bq_download set file_id = '.$file_id.', user_id = '.$user_id.', download_key = "'.$nonce.'"';
		$rows_affected = $wpdb->query ($sql);

		return $rows_affected !== false;
	}

	// for emailed downloads
	public static function finish_download_request ($file_id, $email_id, $download_key)
	{
		global $wpdb;

		$sql = 'update bq_download set time_downloaded = current_timestamp where download_key = "'.$wpdb->_escape ($download_key).'" and file_id = '.$wpdb->escape ($file_id).' and user_id = '.$wpdb->escape ($email_id);
		$rows_affected = $wpdb->query ($sql);

		return $rows_affected !== false;
	}

	public static function download_key_already_exists ($download_key)
	{
		global $wpdb;

		$sql = 'select download_key from bq_download where download_key = "'.$wpdb->_escape ($download_key).'"';
		$result = $wpdb->get_results ($sql, OBJECT); // returns an empty array on error
		return count ($result) != 0;
	}

	public static function download_key_used ($download_key)
	{
		global $wpdb;

		$sql = 'select time_downloaded from bq_download where download_key = "'.$wpdb->_escape ($download_key).'" and time_downloaded <> "0000-00-00 00:00:00"';
		$result = $wpdb->get_results ($sql, OBJECT); // returns an empty array on error
		return count ($result) != 0;
	}

	// for anonymous downloads
	public static function add_download ($file_id, $email_id = null)
	{
		global $wpdb;

		$email_clause = $email_id !== null ? ', user_id = '.$email_id : '';
		$sql = 'insert into bq_download set file_id = '.$wpdb->_escape ($file_id).$email_clause.', time_downloaded = current_timestamp';
		$rows_affected = $wpdb->query ($sql);

		return $rows_affected !== false;
	}

	public static function get_stats ()
	{
		global $wpdb;

		$blog_id = BQDownloadUtil::get_blog_id ();
		$multisite_sql_clause = $blog_id !== null ? 'blog_id = '.$blog_id : 'isnull(blog_id)';

		$server_timezone_str = ini_get ('date.timezone');
		$user_timezone_str = BQdownloadUtil::bq_get_option ('timezone_string');
		$utc_time = time ();
		$server_time = BQdownloadUtil::convert_to_timezone ($utc_time, 'UTC', $server_timezone_str);
		$user_time = BQdownloadUtil::convert_to_timezone ($utc_time, 'UTC', $user_timezone_str);
		$sql_user_timezone = BQdownloadUtil::get_mysql_timezone_str ($user_timezone_str);
		$sql_server_timezone = BQdownloadUtil::get_mysql_timezone_str ($server_timezone_str);

		$chart_period = new BQChartPeriod ('file_downloads');
		switch ($chart_period->get_period ())
		{
			case 'hour':
				// set the end date and time to now
				list ($date, $hour) = explode (' ', date ('Y-m-d H', $user_time));
				list ($year, $month, $day) = explode ('-', $date);
				$user_time_end_date = mktime ($hour, 59, 59, $month, $day, $year); // end of the current hour

				// set the start date and time to 23 hours ago
				list ($date, $hour) = explode (' ', date ('Y-m-d H', $user_time_end_date));
				list ($year, $month, $day) = explode ('-', $date);
				$user_time_start_date = mktime ($hour - 23, 0, 0, $month, $day, $year);

				// create the date strings for the query
				$sql_start_date = date ('Y-m-d H:i:s', $user_time_start_date);
				$sql_end_date = date ('Y-m-d H:i:s', $user_time_end_date);
				$wpdb->query ('SET time_zone = "'.$sql_user_timezone.'"');
				$sql = 'select year(time_downloaded) as year, month(time_downloaded) as month, day(time_downloaded) as day, hour(time_downloaded) as hour, count(time_downloaded) as points '.
							'from bq_download '.
							'inner join bq_download_file '.
								'on bq_download.file_id = bq_download_file.id and '.$multisite_sql_clause.' '.
							'where bq_download_file.deleted = false and time_downloaded >= "'.$sql_start_date.'" and time_downloaded <= "'.$sql_end_date.'" '.
							'group by hour '.
							'order by year, month, day, hour';
				$stats = $wpdb->get_results ($sql, OBJECT); // returns an empty array on error
				$wpdb->query ('SET time_zone = "'.$sql_server_timezone.'"');

				$start_date = $user_time_start_date;

				// get the individual parts of the starting time
				list ($date, $hour) = explode (' ', date ('Y-m-d H', $user_time_start_date));
				list ($year, $month, $day) = explode ('-', $date);

				$stats_return = array ();
				$high_value = 0;
				$result_index = 0;
				$end_hour = $hour + 24;
				for ($h = $hour; $h < $end_hour; $h++)
				{
					// get each value for the next increment
					$date_time = mktime ($h, 0, 0, $month, $day, $year);

					// add the next hour to the list
					if ($stats [$result_index]->hour == date ('H', $date_time))
					{
						$stats_return [] = $stats [$result_index]->points;
						if ($stats [$result_index]->points > $high_value)
							$high_value = $stats [$result_index]->points;
						++$result_index;
					}
					else
						$stats_return [] = 0;
				}
				break;

			case 'day':
				// set the end date and time to now
				list ($year, $month, $day) = explode ('-', date ('Y-m-d', $user_time));
				$user_time_end_date = mktime (23, 59, 59, $month, $day, $year); // end of the current hour

				// set the start date and time to 30 days ago
				list ($year, $month, $day) = explode ('-', date ('Y-m-d', $user_time_end_date));
				$user_time_start_date = mktime (0, 0, 0, $month, $day - 29, $year);

				// create the date strings for the query
				$sql_start_date = date ('Y-m-d H:i:s', $user_time_start_date);
				$sql_end_date = date ('Y-m-d H:i:s', $user_time_end_date);
				$wpdb->query ('SET time_zone = "'.$sql_user_timezone.'"');
				$sql = 'select month(time_downloaded) as month, day(time_downloaded) as day, year(time_downloaded) as year, count(time_downloaded) as points '.
							'from bq_download '.
							'inner join bq_download_file '.
								'on bq_download.file_id = bq_download_file.id and '.$multisite_sql_clause.' '.
							'where bq_download_file.deleted = false and time_downloaded >= "'.$sql_start_date.'" and time_downloaded <= "'.$sql_end_date.'" '.
							'group by year, month, day '.
							'order by year, month, day';
				$stats = $wpdb->get_results ($sql, OBJECT); // returns an empty array on error
				$wpdb->query ('SET time_zone = "'.$sql_server_timezone.'"');

				$start_date = $user_time_start_date;

				// get the individual parts of the starting time
				list ($year, $month, $day) = explode ('-', date ('Y-m-d', $user_time_start_date));

				$stats_return = array ();
				$high_value = 0;
				$result_index = 0;
				$end_day = $day + 30;
				for ($d = $day; $d < $end_day; $d++)
				{
					// get each value for the next increment
					$date_time = mktime (0, 0, 0, $month, $d, $year);

					// add the next day to the list
					if ($stats [$result_index]->year == date ('Y', $date_time) && $stats [$result_index]->month == date ('n', $date_time) && $stats [$result_index]->day == date ('j', $date_time))
					{
						$stats_return [] = $stats [$result_index]->points;
						if ($stats [$result_index]->points > $high_value)
							$high_value = $stats [$result_index]->points;
						++$result_index;
					}
					else
						$stats_return [] = 0;
				}
				break;

			case 'week':
				// set the end date and time to now
				list ($year, $month, $day) = explode ('-', date ('Y-m-d', $user_time));
				$user_time_end_date = mktime (23, 59, 59, $month, $day, $year); // end of the current hour

				// only using week starting Sunday or Monday, if week starts any other day default to Monday
				$start_on_sunday = BQDownloadUtil::bq_get_option ('start_of_week') == 0; // Sun = 0, Sat = 6
				switch (date ('N', $user_time_end_date)) // Mon = 1, Sun = 7
				{
					case 1: $days_this_week = $start_on_sunday ? 2 : 1; break;
					case 2: $days_this_week = $start_on_sunday ? 3 : 2; break;
					case 3: $days_this_week = $start_on_sunday ? 4 : 3; break;
					case 4: $days_this_week = $start_on_sunday ? 5 : 4; break;
					case 5: $days_this_week = $start_on_sunday ? 6 : 5; break;
					case 6: $days_this_week = $start_on_sunday ? 7 : 6; break;
					case 7: $days_this_week = $start_on_sunday ? 1 : 7; break;
				}

				// set the start date and time to 25 weeks before the current week, total = 26 weeks
				list ($year, $month, $day) = explode ('-', date ('Y-m-d', $user_time_end_date));
				$user_time_start_date = mktime (0, 0, 0, $month, ($day - $days_this_week) - 174, $year);

				// determine the week mode for the week function in the query
				// the first week of the year is the first week that contains Sunday or Monday (depending on wordpress setting)
				$week_mode = $start_on_sunday ? 2 : 7;

				// create the date strings for the query
				$sql_start_date = date ('Y-m-d H:i:s', $user_time_start_date);
				$sql_end_date = date ('Y-m-d H:i:s', $user_time_end_date);
				$wpdb->query ('SET time_zone = "'.$sql_user_timezone.'"');
				$sql = 'select week(time_downloaded, '.$week_mode.') as week, count(time_downloaded) as points '.
							'from bq_download '.
							'inner join bq_download_file '.
								'on bq_download.file_id = bq_download_file.id and '.$multisite_sql_clause.' '.
							'where bq_download_file.deleted = false and time_downloaded >= "'.$sql_start_date.'" and time_downloaded <= "'.$sql_end_date.'" '.
							'group by week '.
							'order by time_downloaded';
				$stats = $wpdb->get_results ($sql, OBJECT); // returns an empty array on error
				$start_week_num = $wpdb->get_var ('select week("'.date ('Y-m-d', $user_time_start_date).'", '.$week_mode.')');;
				$highest_week_num = $wpdb->get_var ('select week("'.date ('Y', $user_time_start_date).'-12-31", '.$week_mode.')');
				$wpdb->query ('SET time_zone = "'.$sql_server_timezone.'"');

				// get the week number for the first week in this period
				$start_date = $start_week_num.'|'.$user_time_start_date.'|'.$highest_week_num; // returning week number and timestamp

				$stats_return = array ();
				$high_value = 0;
				$result_index = 0;
				$end_week = $start_week_num + 26;
				for ($w = $start_week_num; $w < $end_week; $w++)
				{
					$week_num = $w;
					if ($week_num > $highest_week_num)
						$week_num -= $highest_week_num;

					// add the next week to the list
					if ($stats [$result_index]->week == $week_num)
					{
						$stats_return [] = $stats [$result_index]->points;
						if ($stats [$result_index]->points > $high_value)
							$high_value = $stats [$result_index]->points;
						++$result_index;
					}
					else
						$stats_return [] = 0;
				}
				break;

			case 'month':
				// set the end date and time to now
				list ($year, $month, $days_in_current_month) = explode ('-', date ('Y-m-t', $user_time));
				$user_time_end_date = mktime (23, 59, 59, $month, $days_in_current_month, $year); // end of the current hour

				// set the start date and time
				list ($year, $month) = explode ('-', date ('Y-n', $user_time_end_date));
				$user_time_start_date = mktime (0, 0, 0, $month - 23, 1, $year);

				// create the date strings for the query
				$sql_start_date = date ('Y-m-d H:i:s', $user_time_start_date);
				$sql_end_date = date ('Y-m-d H:i:s', $user_time_end_date);
				$wpdb->query ('SET time_zone = "'.$sql_user_timezone.'"');
				$sql = 'select year(time_downloaded) as year, month(time_downloaded) as month, count(time_downloaded) as points '.
							'from bq_download '.
							'inner join bq_download_file '.
								'on bq_download.file_id = bq_download_file.id and '.$multisite_sql_clause.' '.
							'where bq_download_file.deleted = false and time_downloaded >= "'.$sql_start_date.'" and time_downloaded <= "'.$sql_end_date.'" '.
							'group by year, month '.
							'order by year, month';
				$stats = $wpdb->get_results ($sql, OBJECT); // returns an empty array on error
				$wpdb->query ('SET time_zone = "'.$sql_server_timezone.'"');

				$start_date = $user_time_start_date;

				// get the individual parts of the starting time
				list ($year, $month, $day) = explode ('-', date ('Y-n-d', $user_time_start_date));

				$stats_return = array ();
				$high_value = 0;
				$result_index = 0;
				$end_month = $month + 24;
				for ($m = $month; $m < $end_month; $m++)
				{
					// get each value for the next increment
					$date_time = mktime (0, 0, 0, $m, $day, $year);

					// add the next month to the list
					if ($stats [$result_index]->year == date ('Y', $date_time) && $stats [$result_index]->month == date ('n', $date_time))
					{
						$stats_return [] = $stats [$result_index]->points;
						if ($stats [$result_index]->points > $high_value)
							$high_value = $stats [$result_index]->points;
						++$result_index;
					}
					else
						$stats_return [] = 0;
				}
				break;
		}

		return array ('high_value' => $high_value, 'starting_at' => $start_date, 'points' => $stats_return);
	}
}
?>
