<?php
// Copyright Brian Quade, BQ Plugins 2012, bqplugins.com

if (!defined ('ABSPATH'))
	die ();

require_once 'bq-download-model.php';
require_once 'bq-pager.php';

class BQDownloadUtil
{
	// singleton data
	protected static $user_id = 0;

	public static function get_readme_wp_version ()
	{
		$dir = plugin_dir_path (__FILE__);
		$readme_txt = realpath ($dir.'/readme.txt');
		if (empty ($readme_txt) || !file_exists ($readme_txt))
			return false;

		$handle = fopen ($readme_txt, 'r');
		$min_version_found = false;
		$min_search_key = 'Requires at least:';
		$min_search_length = strlen ($min_search_key);
		$max_version_found = false;
		$max_search_key = 'Tested up to:';
		$max_search_length = strlen ($max_search_key);
		$more_to_read = true;
		$version = array ('min' => false, 'max' => false);
		do
		{
			$file_line = fgets ($handle);
			$more_to_read = $file_line !== false;
			if ($more_to_read)
			{
				if (!$version ['min'])
				{
					$i = strpos ($file_line, $min_search_key);
					if ($i !== false)
					{
						$value_part = substr ($file_line, $i + $min_search_length);
						$min_arr = sscanf ($value_part, "%s");
						$version ['min'] = trim ($min_arr [0]);
					}
				}

				if (!$version ['max'])
				{
					$i = strpos ($file_line, $max_search_key);
					if ($i !== false)
					{
						$value_part = substr ($file_line, $i + $max_search_length);
						$max_arr = sscanf ($value_part, "%s");
						$version ['max'] = trim ($max_arr [0]);
					}
				}
			}
		}
		while (($version ['min'] === false || $version ['max'] === false) && $more_to_read);
		fclose ($handle);

		if ($version ['min'] === false || $version ['max'] === false)
			return false;

		return $version;
	}

	public static function create_wp_nonce ($action, $detail = null)
	{
		$detail_str = ($detail !== null) ? '_'.$detail : '';
		return wp_create_nonce ('bq-download-ms_'.$action.$detail_str);
	}

	public static function verify_wp_nonce ($nonce, $action, $detail = null)
	{
		$detail_str = ($detail !== null) ? '_'.$detail : '';
		return wp_verify_nonce ($nonce, 'bq-download-ms_'.$action.$detail_str);
	}

	public static function get_blog_id ()
	{
//		return is_multisite () && isset ($GLOBALS ['current_blog']) ? (int) $GLOBALS ['current_blog']->blog_id : null;
		return is_multisite () ? get_current_blog_id () : null;
	}

	public static function create_nonce ($min_length = 64, $max_length = 128)
	{
		$chars = array ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
							'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
							'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z');//,
//							'~', '!', '^', '*', '(', ')', '-', '_', '|');
		$max_char = count ($chars) - 1;
		$nonce = '';
		$length = $min_length < $max_length ? mt_rand ($min_length, $max_length) : $min_length;
		for ($x = 0; $x < $length; $x++)
			$nonce .= $chars [mt_rand (0, $max_char)];
		return $nonce;
	}

	public static function bq_get_user_id ()
	{
		if (self::$user_id == 0)
			self::$user_id = get_current_user_id ();
		return self::$user_id;
	}

	public static function bq_get_option ($option_name, $blog_id = null)
	{
		if (is_multisite ())
		{
			$blog = $blog_id !== null ? $blog_id : self::get_blog_id ();
			$value = get_blog_option ($blog, $option_name);
		}
		else
			$value = get_option ($option_name);
		return $value;
	}

	public static function bq_add_option ($option_name, $option_value, $blog_id = null)
	{
		if (is_multisite ())
		{
			$blog = $blog_id !== null ? $blog_id : self::get_blog_id ();
			add_blog_option ($blog, $option_name, $option_value);
		}
		else
			add_option ($option_name, $option_value);
	}

	public static function bq_update_option ($option_name, $option_value, $blog_id = null)
	{
		if (is_multisite ())
		{
			$blog = $blog_id !== null ? $blog_id : self::get_blog_id ();
			update_blog_option ($blog, $option_name, $option_value);
		}
		else
			update_option ($option_name, $option_value);
	}

	public static function bq_delete_option ($option_name, $blog_id = null)
	{
		if (is_multisite ())
		{
			$blog = $blog_id !== null ? $blog_id : self::get_blog_id ();
			delete_blog_option ($blog, $option_name);
		}
		else
			delete_option ($option_name);
	}

	public static function bq_set_cookie ($cookie_name, $cookie_value)//, $blog_id = null)
	{
		setcookie ($cookie_name, $cookie_value, time () + 252460800, '/');
		$_COOKIE [$cookie_name] = $cookie_value;
	}

	public static function bq_get_cookie ($cookie_name)//, $blog_id = null)
	{
		return isset ($_COOKIE [$cookie_name]) ? $_COOKIE [$cookie_name] : false;
	}

	public static function bq_delete_cookie ($cookie_name)//, $blog_id = null)
	{
		if (is_multisite ())
		{
			$blog = $blog_id !== null ? $blog_id : self::get_blog_id ();
			$cookie_name = $cookie_name.'_'.$blog;
		}

		setcookie ($cookie_name, '', time () - 172800, '/');
	}

	protected static function get_download_dir_name ($blog_id = null)
	{
		if ($blog_id === null)
			$blog_id = self::get_blog_id ();

		$file_dir = 'bq-download-files';
		if ($blog_id !== null)
			$file_dir .= '-'.$blog_id;

		return $file_dir;
	}

	public static function get_download_path_stem ($blog_id = null)
	{
		$doc_root = ABSPATH;
		$add_slash = strcmp ($doc_root [strlen ($doc_root) - 1], '/') != 0 ? '/' : '';
		$file_dir = self::get_download_dir_name ($blog_id);
		return $doc_root.$add_slash.$file_dir;
	}

	// returns:
	// string on success
	// false on failure
	public static function get_download_path ($blog_id = null)
	{
		$download_path = self::bq_get_option ('bq_download_dir', $blog_id);
		$path_was_set = is_string ($download_path);

		$path_stem = self::get_download_path_stem ($blog_id);
		$nonce_path = $path_stem.'-'.self::create_nonce (4, 4).'-'.self::create_nonce (4, 4);

		// version 1.0.1 and earlier
		// setting was empty, nonces were not used in directory names
		$path_was_empty = strlen ($download_path) == 0;
		if ($path_was_set && $path_was_empty)
		{
			$download_path = $nonce_path;

			// make sure the directory exists
			if (file_exists ($path_stem))
				rename ($path_stem, $download_path);
			else
			{
				if (!wp_mkdir_p ($download_path))
					return false;
			}

			// update the setting
			self::bq_update_option ('bq_download_dir', $download_path, $blog_id);
		}

		// make sure the directory exists
		if (!$path_was_set || !file_exists ($download_path)) // first time plugin used, or user deleted the directory?
		{
			$download_path = $nonce_path;
			if (!wp_mkdir_p ($download_path))
				return false;

			// make sure the setting is set
			if ($path_was_set)
				self::bq_update_option ('bq_download_dir', $download_path, $blog_id);
			else
				self::bq_add_option ('bq_download_dir', $download_path, $blog_id);
		}

		return $download_path;
	}

	public static function remove_download_dir ($blog_id = null)
	{
		$dir = self::get_download_path ($blog_id);
		if (file_exists ($dir))
		{
			foreach (scandir ($dir) as $item)
			{
				if (!is_dir ($dir.DIRECTORY_SEPARATOR.$item))
					unlink ($dir.DIRECTORY_SEPARATOR.$item);
			}
			rmdir ($dir);
		}
		self::bq_delete_option ('bq_download_dir', $blog_id);
	}

	// requires datetime in format "YYYY-MM-DD HH:MM:SS"
	// returns date and time formatted to wp settings
	public static function get_wp_date_time ($date_time)
	{
		if (empty ($date_time) || strcmp ($date_time, '0000-00-00 00:00:00') == 0)
			return array ('date' => '', 'time' => '');

		$date_format = self::bq_get_option ('date_format');
		$time_format = self::bq_get_option ('time_format');

		list ($date, $time) = explode (' ', $date_time);
		list ($year, $month, $day) = explode ('-', $date);
		list ($hour, $minute, $second) = explode (':', $time);
		$timestamp = mktime ($hour, $minute, $second, $month, $day, $year);
		$date_text = date ($date_format, $timestamp);
		$time_text = date ($time_format, $timestamp);

		return array ('date' => $date_text, 'time' => $time_text);
	}

	// $datetime can be "YYYY-MM-DD HH:MM:SS" or integer returned from mktime() or time()
	// $from_param defaults to server timezone
	// $to_param defaults to wordpress timezone_string setting
	// returns the same type that was passed, but converted to timezone from wordpress settings
	public static function convert_to_timezone ($datetime, $from_param = null, $to_param = null)
	{
		$timezone_str_from = $from_param !== null ? $from_param : ini_get ('date.timezone');
		$timezone_str_to = $to_param !== null ? $to_param : self::bq_get_option ('timezone_string');
		if (empty ($timezone_str_to) || empty ($timezone_str_from))
			return $datetime;

		if (is_string ($datetime))
		{
			list ($date, $time) = explode (' ', $datetime);
			list ($year, $month, $day) = explode ('-', $date);
			list ($hour, $minute, $second) = explode (':', $time);
			$time_int = mktime ($hour, $minute, $second, $month, $day, $year);
			$time_str = $datetime;
		}
		else if (is_integer ($datetime))
		{
			$time_int = $datetime;
			$time_str = date ('Y-m-d H:i:s', $time_int);
		}

		$to_timezone = new DateTimeZone ($timezone_str_to);
		$from_timezone = new DateTimeZone ($timezone_str_from);
		$myDateTime = new DateTime ($time_str);
		$yourDateTime = new DateTime ($time_str);
		$offset_to = $to_timezone->getOffset ($myDateTime);
		$offset_from = $from_timezone->getOffset ($yourDateTime);
		$offset = $offset_to - $offset_from;

		if (is_string ($datetime))
			return date ('Y-m-d H:i:s', $time_int + $offset);
		else if (is_integer ($datetime))
			return $time_int + $offset;
	}

	public static function get_mysql_timezone_str ($timezone)
	{
		$utc_time = time ();
		$current_time = self::convert_to_timezone ($utc_time, 'UTC', $timezone);

		// get the timezone string for the database
		$timezone_utc_diff = $current_time - $utc_time;
		$tz_raw_hours = $timezone_utc_diff / 3600;
		if ($timezone_utc_diff > 0)
		{
			$tz_hours = floor ($tz_raw_hours);
			$tz_minutes = ($tz_raw_hours - $tz_hours) * 100;
			if ($tz_minutes == 0)
				$tz_minutes = '00';
			$sql_timezone = '+'.$tz_hours.':'.$tz_minutes;
		}
		else if ($timezone_utc_diff < 0)
		{
			$tz_hours = ceil ($tz_raw_hours);
			$tz_minutes = -($tz_raw_hours - $tz_hours) * 100;
			if ($tz_minutes == 0)
				$tz_minutes = '00';
			$sql_timezone = $tz_hours.':'.$tz_minutes;
		}
		else
			$sql_timezone = '+0:00';

		return $sql_timezone;
	}

	public static function get_version_check ()
	{
		ob_start ();
		include dirname (__FILE__).'/bq-version-check.html.php';
		return ob_get_clean ();
	}

	public static function get_settings_page ($error_message = null)
	{
		if ($error_message !== null)
			$bq_page_error = $error_message;
		ob_start ();
		include 'bq-download-settings-page.html.php';
		return ob_get_clean ();
	}

	public static function get_file_page ($error_message = null)
	{
		if ($error_message !== null)
			$bq_page_error = $error_message;
		else
		{
			$stats_window = self::get_download_bar_chart ();
			$file_window = self::get_file_list_window ();
		}
		ob_start ();
		include 'bq-download-file-page.html.php';
		return ob_get_clean ();
	}

	public static function get_email_page ($error_message = null)
	{
		if ($error_message !== null)
			$bq_page_error = $error_message;
		else
			$email_window = self::get_email_list_window ();
		ob_start ();
		include 'bq-download-email-page.html.php';
		return ob_get_clean ();
	}

	public static function get_help_page ($error_message = null)
	{
		ob_start ();
		include 'bq-download-help-page.html.php';
		return ob_get_clean ();
	}

	public static function get_about_page ($error_message = null)
	{
		ob_start ();
		include 'bq-download-about-page.html.php';
		return ob_get_clean ();
	}

	public static function get_download_bar_chart ($specific_stats = null)
	{
		$stats = $specific_stats !== null ? $specific_stats : BQDownloadModel::get_stats ();
		$window_title = 'File Stats';
		ob_start ();
		include 'bq-download-bar-chart.html.php';
		return ob_get_clean ();
	}

	public static function get_file_list_window ($specific_files = null)
	{
		$files = $specific_files !== null ? $specific_files : BQDownloadModel::get_paged_files ();
		ob_start ();
		include 'bq-download-file-window.html.php';
		return ob_get_clean ();
	}

	public static function get_email_list_window ($specific_emails = null)
	{
		$emails = $specific_emails !== null ? $specific_emails : BQDownloadModel::get_users ();
		ob_start ();
		include 'bq-download-email-window.html.php';
		return ob_get_clean ();
	}
}
?>
