<?php
// Copyright Brian Quade, BQ Plugins 2012, bqplugins.com

if (!defined ('ABSPATH'))
	die ();

require_once 'bq-download-model.php';
require_once 'bq-download-util.php';
require_once 'bq-pager.php';
require_once 'bq-sorter.php';
require_once 'bq-chart-period.php';

class BQDownload
{
	protected static $iframe_count = 0;
	protected static $form_count = 0;
	protected static $db_okay = true;

	public static function on_activation ()
	{
	}

	public static function on_load ()
	{
		// verify that required options exist
//		$bq_download_path = BQDownloadUtil::get_download_path ();
//		if (!file_exists ($bq_download_path))
//			BQDownloadUtil::create_download_dir ();

		// return email defaults
		// 1. sendmail return email
		// 2. noreply@blogurl.com
		if (!BQDownloadUtil::bq_get_option ('bq_download_return_email'))
		{
			$site_url = BQDownloadUtil::bq_get_option ('siteurl');
			$url_parts = parse_url ($site_url);
			$domain = $url_parts ['host'];

			$sendmail_from = ini_get ('sendmail_from');
			$bq_download_return_email = !empty ($sendmail_from) ? $sendmail_from : "noreply@$domain";

			BQDownloadUtil::bq_add_option ('bq_download_return_email', $bq_download_return_email);
		}

		// make sure the database schema is correct for this version of the plugin
		self::$db_okay = BQDownloadModel::update_database ();

//self::test_random_downloads ();
//self::test_random_emails ();
//self::test_uninstall ();
	}

	public static function on_init ()
	{
//echo wp_nonce_url ('http://bqplugins.com', 'action').'<br />';
//echo wp_nonce_ays ('action').'<br />';
//echo wp_nonce_field ('action', 'name', true, false).'<br />';
/*
echo wp_create_nonce ('action').'<br />';
echo wp_verify_nonce ('3bd99cc325', 'action').'<br />';
$car_result = check_admin_referer ('action', 'my_arg').'<br />';
echo gettype ($car_result).': '.$car_result.'<br />';
*/
//die;

		// set default values for cookies if they are not set yet
		$file_pager = new BQPager ('file', BQDownloadModel::get_file_count ());
		$email_pager = new BQPager ('email', BQDownloadModel::get_user_count ());
		$file_sorter = new BQSorter ('file');
		$email_sorter = new BQSorter ('email');
		$file_downloads_chart = new BQChartPeriod ('file_downloads');

		// check the get parameters for various commands
		if (isset ($_GET ['x']))
		{
			// download file
			if (strcmp ($_GET ['x'], 'dl') == 0 && isset ($_GET ['f']) && isset ($_GET ['n']))
			{
				// set the variables that are used in download.php
				$file_id = $_GET ['f'];
				$file_name = BQDownloadModel::get_file_name_by_id ($file_id);

				// check for potential old link from unrefreshed page with id of deleted file
				$cross_check_file_id = BQDownloadModel::get_file_id_by_name ($file_name); // returns id of undeleted file
				if ($cross_check_file_id === false || $cross_check_file_id != $file_id)
				{
					ob_start ();
					$iframe_message = $file_name.' has been deleted or changed. Please refresh the page.';
					include dirname (__FILE__).'/bq-iframe-message.html.php';
					die (ob_get_clean ());
				}

				$nonce = $_GET ['n'];
				if (BQDownloadUtil::verify_wp_nonce ($nonce, 'download-file', $file_name.'_'.$file_id) === false)
					die; // failed, don't respond at all

				// these only set when a download key was emailed to the recipient
				if (isset ($_GET ['e']) && isset ($_GET ['k']))
				{
					$email_id = $_GET ['e'];
					$download_key = urldecode ($_GET ['k']);
					$download_key_used = BQDownloadModel::download_key_used ($download_key);
				}

				ob_start ();
				include dirname (__FILE__).'/download.php';
				die (ob_get_clean ());
			}
			// upload file
			else if (strcmp ($_GET ['x'], 'ul') == 0)
			{
				if (isset ($_FILES ['upload_file']))
				{
					// there is a file ready to upload
					$parent_response = new stdClass ();

					$target_dir = realpath (BQDownloadUtil::get_download_path ());
					if (!empty ($target_dir))
					{
						//move_uploaded_file
						$upload_succeeded = move_uploaded_file ($_FILES ['upload_file'] ['tmp_name'], $target_dir.'/'.$_FILES ['upload_file'] ['name']);
						if ($upload_succeeded)
						{
							// add it to the database
							if (BQDownloadModel::add_file ($_FILES ['upload_file'] ['name']) !== false)
								$parent_response->message = $_FILES ['upload_file'] ['name'].' uploaded successfull.';

							// send an updated file list back to the parent window
							$parent_response->file_list = BQDownloadUtil::get_file_list_window ();
						}
						else
							$parent_response->error = 'Failed to upload '.$_FILES ['upload_file'] ['name'].'.';
					}
				}

				$site_url = site_url ();
				ob_start ();
				include dirname (__FILE__).'/upload.php';
				die (ob_get_clean ());
			}
		}
	}

	public static function on_deactivation ()
	{
	}

	// admin menu
	public static function show_settings_page ()
	{
		$bq_page_error = null;

		if (BQDownloadUtil::bq_get_user_id () === 0) // not logged in
			$bq_page_error = 'You are no longer logged in. In order to access this page, you must <a href="'.site_url ().'/wp-admin'.'">login</a>.';

		if ($bq_page_error === null && !self::$db_okay)
			$bq_page_error = 'The database failed to upgrade.';

		echo '<div id="bq_download_admin_settings_page">'.BQDownloadUtil::get_settings_page ($bq_page_error).'</div>';
	}

	public static function show_files_page ()
	{
		if (!self::$db_okay)
		{
			$current_db_version = BQDownloadModel::get_current_db_version ();
			$bq_page_error = 'The database failed to upgrade to version '.$current_db_version.'. The administration page can not be loaded.';
		}
		else
			$bq_page_error = null;
		echo '<div id="bq_download_admin_files_page">'.BQDownloadUtil::get_file_page ($bq_page_error).'</div>';
	}

	public static function show_emails_page ()
	{
		if (!self::$db_okay)
		{
			$current_db_version = BQDownloadModel::get_current_db_version ();
			$bq_page_error = 'The database failed to upgrade to version '.$current_db_version.'. The administration page can not be loaded.';
		}
		else
			$bq_page_error = null;
		echo '<div id="bq_download_admin_emails_page">'.BQDownloadUtil::get_email_page ($bq_page_error).'</div>';
	}

	public static function show_help_page ()
	{
		$bq_page_error = null;

		if (BQDownloadUtil::bq_get_user_id () === 0) // not logged in
			$bq_page_error = 'You are no longer logged in. In order to access this page, you must <a href="'.site_url ().'/wp-admin'.'">login</a>.';

		if ($bq_page_error === null && !self::$db_okay)
			$bq_page_error = 'The database failed to upgrade.';

		echo '<div id="bq_download_admin_help_page">'.BQDownloadUtil::get_help_page ($bq_page_error).'</div>';
	}

	public static function show_about_page ()
	{
		$bq_page_error = null;

		if (BQDownloadUtil::bq_get_user_id () === 0) // not logged in
			$bq_page_error = 'You are no longer logged in. In order to access this page, you must <a href="'.site_url ().'/wp-admin'.'">login</a>.';

		if ($bq_page_error === null && !self::$db_okay)
			$bq_page_error = 'The database failed to upgrade.';

		echo '<div id="bq_download_admin_settings_page">'.BQDownloadUtil::get_about_page ($bq_page_error).'</div>';
	}

	// scripts and styles
	public static function add_admin_inline_scripts ()
	{
		self::add_inline_scripts ();

		ob_start ();
		include dirname (__FILE__).'/bq-download-admin.js.php';
		$bq_download_admin_js = ob_get_clean ();
		echo "\n".'<script type="text/javascript">'."\n".$bq_download_admin_js."\n</script>\n";
	}

	public static function add_admin_script_files ()
	{
		wp_enqueue_style ('bqdownload_style', plugins_url ('', __FILE__).'/bq-download.css');
		wp_enqueue_style ('bqdownload_style_admin', plugins_url ('', __FILE__).'/bq-download-admin.css');

		wp_enqueue_script ('bq_message', plugins_url ('', __FILE__).'/bq-message.js');
		wp_enqueue_script ('bq_ajax', plugins_url ('', __FILE__).'/bq-ajax.js');
//		wp_enqueue_script ('bq_admin_js', plugins_url ('', __FILE__).'/bq-download-admin.js.php');
	}

	public static function add_inline_scripts ()
	{
		ob_start ();
		include dirname (__FILE__).'/bq-download.js.php';
		$js = ob_get_clean ();
		echo "\n".'<script type="text/javascript">'."\n".$js."\n</script>\n";
	}

	public static function add_script_files ()
	{
		wp_enqueue_style ('bqdownload_style', plugins_url ('', __FILE__).'/bq-download.css');

//		wp_enqueue_script ('bq_ajax', plugins_url ('', __FILE__).'/bq-message.js');
		wp_enqueue_script ('bq_ajax', plugins_url ('', __FILE__).'/bq-ajax.js');
	}

	// shortcodes
	public static function bq_download ($atts, $content, $tag)
	{
		// check for missing data
		if (!isset ($atts ['file']) || empty ($atts ['file']))
		{
			$message = 'Must include a file name.';
			ob_start ();
			include dirname (__FILE__).'/error.html.php';
			return ob_get_clean ();
		}

		// make sure it exists before we get it
		$full_path = realpath (BQDownloadUtil::get_download_path ().'/'.$atts ['file']);
		if (empty ($full_path) || !file_exists ($full_path))
		{
			$message = $atts ['file'].' does not exist on the server.';
			ob_start ();
			include dirname (__FILE__).'/error.html.php';
			return ob_get_clean ();
		}

		// default attributes
		if (!isset ($atts ['style']))
			$atts ['style'] = 'button';

		// everything looks okay, prepare to show the download link
		$filename = $atts ['file'];
		$style = $atts ['style'];

		switch ($style)
		{
			case 'emailform':
				$form_email_id = 'bq_download_email_form_'.++self::$form_count;
				$form_email_attachment = isset ($atts ['attach']) && strcasecmp ($atts ['attach'], 'false') != 0;
				ob_start ();
				include dirname (__FILE__).'/bq-download-email-form.html.php';
				return ob_get_clean ();
				break;
			case 'button':
				$color = isset ($atts ['color']) ? $atts ['color'] : 'red';
				$color_array = array ('red', 'green', 'blue');
				switch ($color)
				{
					case 'random':
						$random_color_index = mt_rand (0, count ($color_array) - 1);
						$color = $color_array [$random_color_index];
						break;
					// only allowed colors
//					case 'red':
//					case 'green':
//					case 'blue':
//						break;
					default:
						if (!in_array ($color, $color_array))
							$color = 'red';
						break;
				}
				break;
			case 'link':
				break;
		}

		$iframe_id = 'bq_download_'.++self::$iframe_count;
		$file_id = BQDownloadModel::get_file_id_by_name ($filename);
		ob_start ();
		include dirname (__FILE__).'/bq-download-control.html.php';
		return ob_get_clean ();
	}


private static function test_random_downloads ()
{
	global $wpdb;
	$sql = 'select * from bq_download_file';
	$files = $wpdb->get_results ($sql);
	$file_count = count ($files);
	$day_count = 800;
	if ($file_count > 0)
	{
		for ($f = 0; $f < 1500; $f++)
		{
			$add_file = mt_rand (0, $file_count - 1);
			$now = time () + (60 * 60 * 24); // adding one day
			$second_count = 60 * 60 * 24 * $day_count;
			$download_time = mt_rand ($now - $second_count, $now);
			$time_str = date ('Y-m-d H:i:s', $download_time);
			$sql = 'insert into bq_download set file_id = '.$files [$add_file]->id.', time_downloaded = "'.$time_str.'"';
			$wpdb->query ($sql);
		}
	}
}

private static function test_random_emails ()
{
	$chars = array ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
					'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
					'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z');
	$max_char = count ($chars) - 1;

	for ($f = 0; $f < 57; $f++)
	{
		$name = '';
		$length = mt_rand (4, 24);
		for ($x = 0; $x < $length; $x++)
			$name .= $chars [mt_rand (0, $max_char)];

		$email = $name.'@test.net';
		$user_id = BQDownloadModel::add_user ($email);

		$download_count = mt_rand (1, 5);
		for ($d = 0; $d < $download_count; $d++)
		{
			$download_key = BQDownloadUtil::create_nonce (32, 32);
			$request_made = BQDownloadModel::add_download_request ($email, 'bremen1.jpg', $download_key);

			$download_type = mt_rand (1, 2);
			if ($download_type == 2)
				BQDownloadModel::finish_download_request (1437, $user_id, $download_key);
		}
	}
}

private static function test_uninstall ()
{
	define ('WP_UNINSTALL_PLUGIN', true);
	global $wpdb;
	include dirname (__FILE__).'/uninstall.php';
	die;
}


}
?>
